@@ -6,13 +6,11 @@ from functools import reduce
 from itertools import dropwhile, groupby, islice, product, takewhile, tee, zip_longest
 from random import sample
 from typing import (
-    TYPE_CHECKING,
     Any,
     Callable,
     Collection,
-    Container,
-    ContextManager,
     Deque,
+    Dict,
     Generator,
     Generic,
     Hashable,
@@ -20,7 +18,6 @@ from typing import (
     Iterator,
     List,
     Optional,
-    Sequence,
     Set,
     Tuple,
     Type,
@@ -29,38 +26,41 @@ from typing import (
     overload,
 )
 
-from typing_extensions import Protocol
+from typing_extensions import Concatenate, ParamSpec, Protocol
 
 __all__ = ["flu"]
 
 
 T = TypeVar("T")
 T_co = TypeVar("T_co", covariant=True)
+T_contra = TypeVar("T_contra", contravariant=True)
 _T1 = TypeVar("_T1")
 _T2 = TypeVar("_T2")
 _T3 = TypeVar("_T3")
-
 S = TypeVar("S")
+P = ParamSpec("P")
+
+CallableTakesIterable = Callable[[Iterable[T]], Collection[T]]
 
 
 class SupportsEquality(Protocol):
     def __eq__(self, __other: object) -> bool:
-        ...
+        pass
 
 
 class SupportsGetItem(Protocol[T_co]):
-    def __getitem__(self, __k: Hashable) -> T:
-        ...
+    def __getitem__(self, __k: Hashable) -> T_co:
+        pass
 
 
 class SupportsIteration(Protocol[T_co]):
     def __iter__(self) -> Iterator[T]:
-        ...
+        pass
 
 
 class SupportsLessThan(Protocol):
     def __lt__(self, __other: Any) -> bool:
-        ...
+        pass
 
 
 SupportsLessThanT = TypeVar("SupportsLessThanT", bound="SupportsLessThan")
@@ -74,21 +74,18 @@ def identity(x: T) -> T:
     return x
 
 
-CallableTakesIterable = Callable[[Iterable[T]], Collection]
-
-
 class Fluent(Generic[T]):
     """A fluent interface to lazy generator functions
 
     >>> from flupy import flu
     >>> (
-            flu(range(100))
-            .map(lambda x: x**2)
-            .filter(lambda x: x % 3 == 0)
-            .chunk(3)
-            .take(2)
-            .collect()
-        )
+        flu(range(100))
+        .map(lambda x: x**2)
+        .filter(lambda x: x % 3 == 0)
+        .chunk(3)
+        .take(2)
+        .to_list()
+    )
     [[0, 9, 36], [81, 144, 225]]
     """
 
@@ -113,13 +110,13 @@ class Fluent(Generic[T]):
         elif isinstance(key, slice):
             return flu(islice(self._iterator, key.start, key.stop, key.step))
         else:
-            raise KeyError("Key must be non-negative integer or slice, not {}".format(key))
+            raise TypeError(f"Indices must be non-negative integers or slices, not {type(key).__name__}")
 
     ### Summary ###
-    def collect(self, n: int = None, container_type: CallableTakesIterable[T] = list) -> Collection[T]:
+    def collect(self, n: Optional[int] = None, container_type: CallableTakesIterable[T] = list) -> Collection[T]:
         """Collect items from iterable into a container
 
-        >>> flu(range(4).collect()
+        >>> flu(range(4)).collect()
         [0, 1, 2, 3]
 
         >>> flu(range(4)).collect(container_type=set)
@@ -130,6 +127,14 @@ class Fluent(Generic[T]):
         """
         return container_type(self.take(n))
 
+    def to_list(self) -> List[T]:
+        """Collect items from iterable into a list
+
+        >>> flu(range(4)).to_list()
+        [0, 1, 2, 3]
+        """
+        return list(self)
+
     def sum(self) -> Union[T, int]:
         """Sum of elements in the iterable
 
@@ -137,7 +142,7 @@ class Fluent(Generic[T]):
         6
 
         """
-        return sum(self)
+        return sum(self)  # type: ignore
 
     def count(self) -> int:
         """Count of elements in the iterable
@@ -164,21 +169,19 @@ class Fluent(Generic[T]):
         return max(self)
 
     def first(self, default: Any = Empty()) -> T:
-        """Return the first item of the iterable. Raise IndexError if empty or default if provided.
+        """Return the first item of the iterable. Raise IndexError if empty, or return default if provided.
 
-               >>> flu([0, 1, 2, 3]).first()
-               0
-               >>> flu([]).first(default='some_default')
-               'some default'
-
-        when *default* is not provided and the iterable is empty, raise IndexError
+        >>> flu([0, 1, 2, 3]).first()
+        0
+        >>> flu([]).first(default="some_default")
+        'some_default'
         """
-        x = default
+        x: Union[Empty, T] = default
         for x in self:
             return x
         if isinstance(x, Empty):
             raise IndexError("Empty iterator")
-        return default
+        return x
 
     def last(self, default: Any = Empty()) -> T:
         """Return the last item of the iterble. Raise IndexError if empty or default if provided.
@@ -186,7 +189,7 @@ class Fluent(Generic[T]):
         >>> flu([0, 1, 2, 3]).last()
         3
         >>> flu([]).last(default='some_default')
-        'some default'
+        'some_default'
         """
         x: Union[Empty, T] = default
         for x in self:
@@ -216,7 +219,7 @@ class Fluent(Generic[T]):
         [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
 
         >>> flu(range(15)).tail(n=2)
-        [18, 19]
+        [13, 14]
         """
         val: Union[List[Empty], Tuple[Any, ...]] = [Empty()]
         for val in self.window(n, fill_value=Empty()):
@@ -235,14 +238,14 @@ class Fluent(Generic[T]):
 
         Note: sorting loads the entire iterable into memory
 
-               >>> flu([3,6,1]).sort().collect()
-               [1, 3, 6]
+        >>> flu([3,6,1]).sort().to_list()
+        [1, 3, 6]
 
-               >>> flu([3,6,1]).sort(reverse=True).collect()
-               [6, 3, 1]
+        >>> flu([3,6,1]).sort(reverse=True).to_list()
+        [6, 3, 1]
 
-               >>> flu([3,-6,1]).sort(key=abs).collect()
-               [1, 3, -6]
+        >>> flu([3,-6,1]).sort(key=abs).to_list()
+        [1, 3, -6]
         """
         return Fluent(sorted(self, key=key, reverse=reverse))
 
@@ -258,8 +261,8 @@ class Fluent(Generic[T]):
 
         Note: join_left loads *other* into memory
 
-               >>> flu(range(6)).join_left(range(0, 6, 2)).collect()
-               [(0, 0), (1, None), (2, 2), (3, None), (4, 4), (5, None)]
+        >>> flu(range(6)).join_left(range(0, 6, 2)).to_list()
+        [(0, 0), (1, None), (2, 2), (3, None), (4, 4), (5, None)]
         """
 
         def _impl() -> Generator[Tuple[T, Union[_T1, None]], None, None]:
@@ -292,8 +295,8 @@ class Fluent(Generic[T]):
 
         Note: join_inner loads *other* into memory
 
-               >>> flu(range(6)).join_inner(range(0, 6, 2)).collect()
-               [(0, 0), (2, 2), (4, 4)]
+        >>> flu(range(6)).join_inner(range(0, 6, 2)).to_list()
+        [(0, 0), (2, 2), (4, 4)]
 
         """
 
@@ -312,15 +315,65 @@ class Fluent(Generic[T]):
 
         return Fluent(_impl())
 
+    def join_full(
+        self,
+        other: Iterable[_T1],
+        key: Callable[[T], Hashable] = identity,
+        other_key: Callable[[_T1], Hashable] = identity,
+    ) -> "Fluent[Tuple[Union[T, None], Union[_T1, None]]]":
+        """Join the iterable with another iterable using equality between *key* applied to self and *other_key* applied to *other* to identify matching entries
+
+        Returns all entries from both iterables. When no matching entry is found, entries are paired with None
+
+        Note: join_full loads both *self* and *other* into memory
+
+        >>> flu(range(4)).join_full(range(2, 6)).to_list()
+        [(0, None), (1, None), (2, 2), (3, 3), (None, 4), (None, 5)]
+        """
+
+        def _impl() -> Generator[Tuple[Union[T, None], Union[_T1, None]], None, None]:
+
+            # Build lookup for other
+            other_lookup: Dict[Hashable, List[_T1]] = defaultdict(list)
+            other_keys_seen: Set[Hashable] = set()
+
+            for entry_other in other:
+                other_key_val = other_key(entry_other)
+                other_lookup[other_key_val].append(entry_other)
+                other_keys_seen.add(other_key_val)
+
+            # Track which keys from other have been matched
+            matched_other_keys: Set[Hashable] = set()
+
+            # Process all entries from self
+            for entry in self:
+                entry_key = key(entry)
+                matches: Optional[List[_T1]] = other_lookup.get(entry_key)
+
+                if matches:
+                    matched_other_keys.add(entry_key)
+                    for match in matches:
+                        yield (entry, match)
+                else:
+                    yield (entry, None)
+
+            # Yield unmatched entries from other
+            unmatched_keys = other_keys_seen - matched_other_keys
+            for unmatched_key in unmatched_keys:
+                for entry_other in other_lookup[unmatched_key]:
+                    yield (None, entry_other)
+
+        return Fluent(_impl())
+
     def shuffle(self) -> "Fluent[T]":
         """Randomize the order of elements in the interable
 
         Note: shuffle loads the entire iterable into memory
 
-               >>> flu([3,6,1]).shuffle().collect()
-               [6, 1, 3]
+        >>> flu([3,6,1]).shuffle().to_list()
+        [6, 1, 3]
         """
-        dat: List[T] = list(self.collect())
+        dat: List[T] = self.to_list()
         return Fluent(sample(dat, len(dat)))
 
     def group_by(
@@ -332,23 +385,24 @@ class Fluent(Generic[T]):
 
         When the iterable is pre-sorted according to *key*, setting *sort* to False will prevent loading the dataset into memory and improve performance
 
-               >>> flu([2, 4, 2, 4]).group_by().collect()
-               [2, <flu object>), (4, <flu object>)]
+        >>> flu([2, 4, 2, 4]).group_by().to_list()
+        [(2, <flu object>), (4, <flu object>)]
 
         Or, if the iterable is pre-sorted
 
-               >>> flu([2, 2, 5, 5]).group_by(sort=False).collect()
-               [(2, <flu object>), (5, <flu object>)]
+        >>> flu([2, 2, 5, 5]).group_by(sort=False).to_list()
+        [(2, <flu object>), (5, <flu object>)]
 
         Using a key function
-               >>>  points = [
-                        {'x': 1, 'y': 0},
-                        {'x': 4, 'y': 3},
-                        {'x': 1, 'y': 5}
-                    ]
-               >>> key_func = lambda u: u['x']
-               >>> flu(points).group_by(key=key_func, sort=True).collect()
-               [(1, <flu object>), (4, <flu object>)]
+
+        >>> points = [
+            {'x': 1, 'y': 0},
+            {'x': 4, 'y': 3},
+            {'x': 1, 'y': 5}
+        ]
+        >>> key_func = lambda u: u['x']
+        >>> flu(points).group_by(key=key_func, sort=True).to_list()
+        [(1, <flu object>), (4, <flu object>)]
         """
 
         gen = self.sort(key) if sort else self
@@ -357,10 +411,10 @@ class Fluent(Generic[T]):
     def unique(self, key: Callable[[T], Hashable] = identity) -> "Fluent[T]":
         """Yield elements that are unique by a *key*.
 
-        >>> flu([2, 3, 2, 3]).unique().collect()
+        >>> flu([2, 3, 2, 3]).unique().to_list()
         [2, 3]
 
-        >>> flu([2, -3, -2, 3]).unique(key=abs).collect()
+        >>> flu([2, -3, -2, 3]).unique(key=abs).to_list()
         [2, -3]
         """
 
@@ -384,8 +438,8 @@ class Fluent(Generic[T]):
 
         >>> import time
         >>> start_time = time.time()
-        >>> flu(range(3)).rate_limit(3).collect()
-        >>> print('Runtime', time.time() - start_time)
+        >>> _ = flu(range(3)).rate_limit(3).to_list()
+        >>> print('Runtime', int(time.time() - start_time))
         1.00126 # approximately 1 second for 3 items
         """
 
@@ -411,10 +465,10 @@ class Fluent(Generic[T]):
         and after iteration ends respectively. Each will be called exactly once.
 
 
-            >>> flu(range(2)).side_effect(lambda x: print(f'Collected {x}')).collect()
-            Collected 0
-            Collected 1
-            [0, 1]
+        >>> flu(range(2)).side_effect(lambda x: print(f'Collected {x}')).to_list()
+        Collected 0
+        Collected 1
+        [0, 1]
         """
 
         def _impl() -> Generator[T, None, None]:
@@ -434,31 +488,30 @@ class Fluent(Generic[T]):
 
     ### End Side Effect ###
 
-    def map(self, func: Callable[[T], _T1], *args: Any, **kwargs: Any) -> "Fluent[_T1]":
+    def map(self, func: Callable[Concatenate[T, P], _T1], *args: Any, **kwargs: Any) -> "Fluent[_T1]":
         """Apply *func* to each element of iterable
 
-        >>> flu(range(5)).map(lambda x: x*x).collect()
+        >>> flu(range(5)).map(lambda x: x*x).to_list()
         [0, 1, 4, 9, 16]
         """
 
         def _impl() -> Generator[_T1, None, None]:
             for val in self._iterator:
-                # Not possible to type Callabe[[T, args, kwargs], _T1]
-                yield func(val, *args, **kwargs)  # type: ignore
+                yield func(val, *args, **kwargs)
 
         return Fluent(_impl())
 
-    def map_item(self: "Fluent[SupportsGetItem[T]]", item: Hashable) -> "Fluent[SupportsGetItem[T]]":
+    def map_item(self: "Fluent[SupportsGetItem[T]]", item: Hashable) -> "Fluent[T]":
         """Extracts *item* from every element of the iterable
 
-        >>> flu([(2, 4), (2, 5)]).map_item(1).collect()
+        >>> flu([(2, 4), (2, 5)]).map_item(1).to_list()
         [4, 5]
 
-        >>> flu([{'mykey': 8}, {'mykey': 5}]).map_item('mykey').collect()
+        >>> flu([{'mykey': 8}, {'mykey': 5}]).map_item('mykey').to_list()
         [8, 5]
         """
 
-        def _impl() -> Generator[SupportsGetItem[T], None, None]:
+        def _impl() -> Generator[T, None, None]:
             for x in self:
                 yield x[item]
 
@@ -469,15 +522,15 @@ class Fluent(Generic[T]):
 
         >>> from collections import namedtuple
         >>> MyTup = namedtuple('MyTup', ['value', 'backup_val'])
-        >>> flu([MyTup(1, 5), MyTup(2, 4)]).map_attr('value').collect()
+        >>> flu([MyTup(1, 5), MyTup(2, 4)]).map_attr('value').to_list()
         [1, 2]
         """
         return self.map(lambda x: getattr(x, attr))
 
-    def filter(self, func: Callable[..., bool], *args: Any, **kwargs: Any) -> "Fluent[T]":
+    def filter(self, func: Callable[Concatenate[T, P], bool], *args: Any, **kwargs: Any) -> "Fluent[T]":
         """Yield elements of iterable where *func* returns truthy
 
-        >>> flu(range(10)).filter(lambda x: x % 2 == 0).collect()
+        >>> flu(range(10)).filter(lambda x: x % 2 == 0).to_list()
         [0, 2, 4, 6, 8]
         """
 
@@ -492,8 +545,8 @@ class Fluent(Generic[T]):
         """Apply a function of two arguments cumulatively to the items of the iterable,
         from left to right, so as to reduce the sequence to a single value
 
-            >>> flu(range(5)).reduce(lambda x, y: x + y)
-            10
+        >>> flu(range(5)).reduce(lambda x, y: x + y)
+        10
         """
         return reduce(func, self)
 
@@ -502,24 +555,21 @@ class Fluent(Generic[T]):
         from left to right, starting with *initial*, so as to fold the sequence to
         a single value
 
-            >>> flu(range(5)).fold_left(lambda x, y: x + str(y), "")
-            "01234"
+        >>> flu(range(5)).fold_left(lambda x, y: x + str(y), "")
+        '01234'
         """
         return reduce(func, self, initial)
 
     @overload
-    def zip(self, __iter1: Iterable[_T1]) -> "Fluent[Tuple[T, _T1]]":
-        ...
+    def zip(self, __iter1: Iterable[_T1]) -> "Fluent[Tuple[T, _T1]]": ...
 
     @overload
-    def zip(self, __iter1: Iterable[_T1], __iter2: Iterable[_T2]) -> "Fluent[Tuple[T, _T1, _T2]]":
-        ...
+    def zip(self, __iter1: Iterable[_T1], __iter2: Iterable[_T2]) -> "Fluent[Tuple[T, _T1, _T2]]": ...
 
     @overload
     def zip(
         self, __iter1: Iterable[_T1], __iter2: Iterable[_T2], __iter3: Iterable[_T3]
-    ) -> "Fluent[Tuple[T, _T1, _T2, _T3]]":
-        ...
+    ) -> "Fluent[Tuple[T, _T1, _T2, _T3]]": ...
 
     @overload
     def zip(
@@ -528,23 +578,20 @@ class Fluent(Generic[T]):
         __iter2: Iterable[Any],
         __iter3: Iterable[Any],
         __iter4: Iterable[Any],
-        *iterable: Iterable[Any]
-    ) -> "Fluent[Tuple[T, ...]]":
-        ...
+        *iterable: Iterable[Any],
+    ) -> "Fluent[Tuple[T, ...]]": ...
 
-    def zip(
-        self, *iterable: Iterable[Any]
-    ) -> Union[
+    def zip(self, *iterable: Iterable[Any]) -> Union[
         "Fluent[Tuple[T, ...]]",
         "Fluent[Tuple[T, _T1]]",
         "Fluent[Tuple[T, _T1, _T2]]",
         "Fluent[Tuple[T, _T1, _T2, _T3]]",
     ]:
         """Yields tuples containing the i-th element from the i-th
-        argument in the chainable, and the iterable
+        argument in the instance, and the iterable
 
-            >>> flu(range(5)).zip(range(3, 0, -1)).collect()
-            [(0, 3), (1, 2), (2, 1)]
+        >>> flu(range(5)).zip(range(3, 0, -1)).to_list()
+        [(0, 3), (1, 2), (2, 1)]
         """
         # @self_to_flu is not compatible with @overload
         # make sure any usage of self supports arbitrary iterables
@@ -553,32 +600,32 @@ class Fluent(Generic[T]):
 
     def zip_longest(self, *iterable: Iterable[_T1], fill_value: Any = None) -> "Fluent[Tuple[T, ...]]":
         """Yields tuples containing the i-th element from the i-th
-        argument in the chainable, and the iterable
+        argument in the instance, and the iterable
         Iteration continues until the longest iterable is exhaused.
         If iterables are uneven in length, missing values are filled in with fill value
 
-            >>> flu(range(5)).zip_longest(range(3, 0, -1)).collect()
-            [(0, 3), (1, 2), (2, 1), (3, None), (4, None)]
+        >>> flu(range(5)).zip_longest(range(3, 0, -1)).to_list()
+        [(0, 3), (1, 2), (2, 1), (3, None), (4, None)]
 
 
-            >>> flu(range(5)).zip_longest(range(3, 0, -1), fill_value='a').collect()
-            [(0, 3), (1, 2), (2, 1), (3, 'a'), (4, 'a')]
+        >>> flu(range(5)).zip_longest(range(3, 0, -1), fill_value='a').to_list()
+        [(0, 3), (1, 2), (2, 1), (3, 'a'), (4, 'a')]
         """
         return Fluent(zip_longest(self, *iterable, fillvalue=fill_value))
 
     def enumerate(self, start: int = 0) -> "Fluent[Tuple[int, T]]":
-        """Yields tuples from the chainable where the first element
+        """Yields tuples from the instance where the first element
         is a count from initial value *start*.
 
-            >>> flu(range(5)).zip_longest(range(3, 0, -1)).collect()
-            [(0, 3), (1, 2), (2, 1), (3, None), (4, None)]
+        >>> flu([3,4,5]).enumerate().to_list()
+        [(0, 3), (1, 4), (2, 5)]
         """
         return Fluent(enumerate(self, start=start))
 
     def take(self, n: Optional[int] = None) -> "Fluent[T]":
         """Yield first *n* items of the iterable
 
-        >>> flu(range(10)).take(2).collect()
+        >>> flu(range(10)).take(2).to_list()
         [0, 1]
         """
         return Fluent(islice(self._iterator, n))
@@ -586,7 +633,7 @@ class Fluent(Generic[T]):
     def take_while(self, predicate: Callable[[T], bool]) -> "Fluent[T]":
         """Yield elements from the chainable so long as the predicate is true
 
-        >>> flu(range(10)).take_while(lambda x: x < 3).collect()
+        >>> flu(range(10)).take_while(lambda x: x < 3).to_list()
         [0, 1, 2]
         """
         return Fluent(takewhile(predicate, self._iterator))
@@ -595,8 +642,8 @@ class Fluent(Generic[T]):
         """Drop elements from the chainable as long as the predicate is true;
         afterwards, return every element
 
-            >>> flu(range(10)).drop_while(lambda x: x < 3).collect()
-            [4, 5, 6, 7, 8, 9]
+        >>> flu(range(10)).drop_while(lambda x: x < 3).to_list()
+        [3, 4, 5, 6, 7, 8, 9]
         """
         return Fluent(dropwhile(predicate, self._iterator))
 
@@ -605,8 +652,8 @@ class Fluent(Generic[T]):
 
         if the iterable is not evenly divisiible by *n*, the final list will be shorter
 
-            >>> flu(range(10)).chunk(3).collect()
-            [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
+        >>> flu(range(10)).chunk(3).to_list()
+        [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
         """
 
         def _impl() -> Generator[List[T], None, None]:
@@ -623,7 +670,7 @@ class Fluent(Generic[T]):
     def flatten(
         self,
         depth: int = 1,
-        base_type: Type[object] = None,
+        base_type: Optional[Type[object]] = None,
         iterate_strings: bool = False,
     ) -> "Fluent[Any]":
         """Recursively flatten nested iterables (e.g., a list of lists of tuples)
@@ -632,23 +679,23 @@ class Fluent(Generic[T]):
         Strings are treated as non-iterable for convenience. set iterate_string=True
         to change that behavior.
 
-            >>> flu([[0, 1, 2], [3, 4, 5]]).flatten().collect()
-            [0, 1, 2, 3, 4, 5]
+        >>> flu([[0, 1, 2], [3, 4, 5]]).flatten().to_list()
+        [0, 1, 2, 3, 4, 5]
 
-            >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten().collect()
-            [0, [1, 2], [3, 4], 5]
+        >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten().to_list()
+        [0, [1, 2], [3, 4], 5]
 
-            >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten(depth=2).collect()
-            [0, 1, 2, 3, 4, 5]
+        >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten(depth=2).to_list()
+        [0, 1, 2, 3, 4, 5]
 
-            >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten(depth=2).collect()
-            [0, 1, 2, 3, 4, 5]
+        >>> flu([[0, [1, 2]], [[3, 4], 5]]).flatten(depth=2).to_list()
+        [0, 1, 2, 3, 4, 5]
 
-            >>> flu([1, (2, 2), 4, [5, (6, 6, 6)]]).flatten(base_type=tuple).collect()
-            [1, (2, 2), 4, 5, (6, 6, 6)]
+        >>> flu([1, (2, 2), 4, [5, (6, 6, 6)]]).flatten(base_type=tuple).to_list()
+        [1, (2, 2), 4, 5, (6, 6, 6)]
 
-            >>> flu([[2, 0], 'abc', 3, [4]]).flatten(iterate_strings=True).collect()
-            [2, 0, 'a', 'b', 'c', 3, 4]
+        >>> flu([[2, 0], 'abc', 3, [4]]).flatten(iterate_strings=True).to_list()
+        [2, 0, 'a', 'b', 'c', 3, 4]
         """
 
         # TODO(OR): Reimplement with strong types
@@ -675,24 +722,13 @@ class Fluent(Generic[T]):
     def denormalize(self: "Fluent[SupportsIteration[Any]]", iterate_strings: bool = False) -> "Fluent[Tuple[Any, ...]]":
         """Denormalize iterable components of each record
 
-        >>> flu([("abc", [1, 2, 3])]).denormalize().collect()
-        [("abc", 1), ("abc", 2), ("abc", 3)]
+        >>> flu([("abc", [1, 2, 3])]).denormalize().to_list()
+        [('abc', 1), ('abc', 2), ('abc', 3)]
 
-        >>> flu([("abc", [1, 2, 3])]).denormalize(iterate_strings=True).collect()
-        [
-            ("a", 1),
-            ("a", 2),
-            ("a", 3),
-            ("b", 1),
-            ("b", 2),
-            ("b", 3),
-            ("c", 1),
-            ("c", 2),
-            ("c", 3),
-        ]
+        >>> flu([("abc", [1, 2])]).denormalize(iterate_strings=True).to_list()
+        [('a', 1), ('a', 2), ('b', 1), ('b', 2), ('c', 1), ('c', 2)]
 
-
-        >>> flu([("abc", [])]).denormalize().collect()
+        >>> flu([("abc", [])]).denormalize().to_list()
         []
         """
 
@@ -731,17 +767,17 @@ class Fluent(Generic[T]):
         If the length of the iterable does not evenly divide by the *step*
         the final output is padded with *fill_value*
 
-            >>> flu(range(5)).window(3).collect()
-            [(0, 1, 2), (1, 2, 3), (2, 3, 4)]
+        >>> flu(range(5)).window(3).to_list()
+        [(0, 1, 2), (1, 2, 3), (2, 3, 4)]
 
-            >>> flu(range(5)).window(n=3, step=2).collect()
-            [(0, 1, 2), (1, 2, 3), (2, 3, 4)]
+        >>> flu(range(5)).window(n=3, step=2).to_list()
+        [(0, 1, 2), (2, 3, 4)]
 
-            >>> flu(range(9)).window(n=4, step=3).collect()
-            [(0, 1, 2, 3), (3, 4, 5, 6), (6, 7, 8, None)]
+        >>> flu(range(9)).window(n=4, step=3).to_list()
+        [(0, 1, 2, 3), (3, 4, 5, 6), (6, 7, 8, None)]
 
-            >>> flu(range(9)).window(n=4, step=3, fill_value=-1).collect()
-            [(0, 1, 2, 3), (3, 4, 5, 6), (6, 7, 8, -1)]
+        >>> flu(range(9)).window(n=4, step=3, fill_value=-1).to_list()
+        [(0, 1, 2, 3), (3, 4, 5, 6), (6, 7, 8, -1)]
         """
 
         def _impl() -> Generator[Tuple[Any, ...], None, None]:
@@ -791,11 +827,11 @@ class Fluent(Generic[T]):
         anywhere else; otherwise, the iterable could get advanced without the
         tee objects being informed
 
-            >>> copy1, copy2 = flu(range(5)).tee()
-            >>> copy1.sum()
-            10
-            >>> copy2.collect()
-            [0, 1, 2, 3, 4]
+        >>> copy1, copy2 = flu(range(5)).tee()
+        >>> copy1.sum()
+        10
+        >>> copy2.to_list()
+        [0, 1, 2, 3, 4]
         """
         return Fluent((Fluent(x) for x in tee(self, n)))
 
@@ -810,7 +846,7 @@ class flu(Fluent[T]):
             .filter(lambda x: x % 3 == 0)
             .chunk(3)
             .take(2)
-            .collect()
+            .to_list()
         )
     [[0, 9, 36], [81, 144, 225]]
     """
